@Generated(
    value = {
        "Ogma: Database Access Code Generator for Java {{compiler_version}}"
    },
    date = "{{datetime}}",
    comments = "This is generated code (Ogma {{compiler_version}})"
)
