/*
 * GENERATED FILE: {{file_name}}
 *
 * THIS FILE HAS BEEN GENERATED FROM THE DATABASE MODEL FILE:
 * {{database_model_file}}
 *
 * DO NOT EDIT THIS FILE DIRECTLY
 *
 * DO NOT INCLUDE THIS FILE IN SOURCE CONTROL
 *
 */
